%
% process the output of parse_ohsumed.sh
%

prefix = 'ohsumed'
filename = [prefix '_dict'];
fid = fopen(filename);
currline = fgetl(fid);
fclose(fid);
t0 = findstr(currline,' ');

[word dummy] = strread(currline(t0(3)+1:end),'%s %d');
a = zeros(length(word),1);
fprintf('there are %d words in the corpus',length(word));

save([prefix '_dict'],'word');

Nwords = length(word);
Ndocuments = 60000;


counts = sparse(Nwords,Ndocuments);
%
% different clases
%
classes = {'C01','C02','C03','C04','C05','C06','C07','C08','C09','C10','C11','C12','C13','C14','C15','C16','C17','C18','C19','C20','C21','C22','C23'};

labels = zeros(Ndocuments,length(classes));

knownIDs = zeros(Ndocuments,1);

pattern = '/([\w-]*)/0';

filename = [prefix '_counts'];
fid = fopen(filename);

cnt = 0;
emptydoc = 0;
while 1
    currline = fgetl(fid);
    if ~ischar(currline), break, end
    
    t2 = findstr(currline,'/');
    t = findstr(currline,' ');
    
    docID = str2num(currline(t2(2)+1:t(1)));
    
    docfilename = currline(1:t(1)-1);
    n = regexp(docfilename,pattern,'tokens');
    t_class = n{1}{1};
    groupname = currline(t(1)+1:t(2)-1);
    [dummy,t_label] = max(strcmp(groupname,classes));

    [wordid wordcount] = strread(currline(t(3)+1:end),'%d %d');

    if ~prod(size(wordcount)) | ~prod(size(wordid))
	emptydoc = emptydoc+1;
	fprintf('empty document no %d\n',emptydoc);
	continue;
    end

    cnt = cnt+1;
    knownIDs(cnt) = docID;
    counts(wordid+1,cnt) = wordcount;
    [ignore,t_label] = max(ismember(classes,t_class));


    a = knownIDs == docID;
    labels(a,t_label) = 1;

    
    if ~mod(cnt,1000), disp('.');end
end

fclose(fid);
% remove all empty documents at the end (allocated more mem than
% necessary)
counts(:,cnt+1:end) = [];
labels(cnt+1:end,:) = [];
knownIDs(cnt+1:end) = [];

% remove those documents which appear more than once in the corpus
cnt = 0;
while 1
  cnt = cnt + 1;
  if cnt > size(knownIDs,1)
    break;
  end
  
  a = false(size(knownIDs,1),1);
  a(cnt+1:end) = knownIDs(cnt) == knownIDs(cnt+1:end);
  
  knownIDs(a) = [];
  counts(:,a) = [];
  labels(a,:) = [];
  
  if ~mod(cnt,200)
    fprintf('%d / %d\n',cnt,size(knownIDs,1));
  end

end



save([prefix '_counts'],'counts','labels','knownIDs','-v6');
save('MultiClass','knownIDs','labels');




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MAKE A TRAIN/TEST SPLIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if 0
Ntest = ceil(Ntrain/2);
Ntrain = Ntrain - Ntest;

shuff = randperm(size(counts,2));

ytrain = labels(shuff(1:Ntrain));
Xtrain = counts(:,shuff(1:Ntrain));
ytest = labels(shuff(Ntrain+1:end));
Xtest = counts(:,shuff(Ntrain+1:end));

save('Ohsumed_split','Xtrain','Xtest','ytrain','ytest','shuff','-v6');

end
